library fancy_bottom_navigation;

import 'dart:async';

import 'package:flutter/material.dart';
import './paint/half_painter.dart';
import './paint/half_clipper.dart';
import './internal/indzooom_tab_item.dart';
import 'dart:io';

const double CIRCLE_SIZE = 60;
const double ARC_HEIGHT = 70;
const double ARC_WIDTH = 90;
const double CIRCLE_OUTLINE = 10;
const double SHADOW_ALLOWANCE = 20;
const double BAR_HEIGHT = 60;

class IndzooomFancyBottomNavigation extends StatefulWidget {
  IndzooomFancyBottomNavigation(
      {@required this.tabs,
      @required this.onTabChangedListener,
      this.key,
      this.initialSelection = 0,
      this.circleColor,
      this.activeIconColor,
      this.inactiveIconColor,
      this.textColor,
      this.titleSize,
      this.barBackgroundColor})
      : assert(onTabChangedListener != null),
        assert(tabs != null),
        assert(tabs.length > 1 && tabs.length < 6);

  final Color activeIconColor;
  final Color barBackgroundColor;
  final Color circleColor;
  final Color inactiveIconColor;
  final int initialSelection;
  final Key key;
  final List<TabData> tabs;
  final Color textColor;
  final double titleSize;

  @override
  IndzooomFancyBottomNavigationState createState() => IndzooomFancyBottomNavigationState();

  final Function(int position) onTabChangedListener;
}

class IndzooomFancyBottomNavigationState extends State<IndzooomFancyBottomNavigation>
    with TickerProviderStateMixin, RouteAware {
  IconData activeIcon = Icons.search;
  Color activeIconColor;
  Color barBackgroundColor;
  Color circleColor;
  int currentSelected = 0;
  Color inactiveIconColor;
  IconData nextIcon = Icons.search;
  Color textColor;
  double titleSize;

  double _circleAlignX = 0;
  double _circleIconAlpha = 1;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    activeIcon = widget.tabs[currentSelected].iconData;

    circleColor = (widget.circleColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
            ? Colors.white
            : Theme.of(context).primaryColor
        : widget.circleColor;

    activeIconColor = (widget.activeIconColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
            ? Colors.black54
            : Colors.white
        : widget.activeIconColor;

    barBackgroundColor = (widget.barBackgroundColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
            ? Color(0xFF212121)
            : Colors.white
        : widget.barBackgroundColor;

    textColor = (widget.textColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
            ? Colors.white
            : Colors.black54
        : widget.textColor;

    titleSize = (widget.titleSize == null)
        ? 12.0
        : widget.titleSize;

    inactiveIconColor = (widget.inactiveIconColor == null)
        ? (Theme.of(context).brightness == Brightness.dark)
            ? Colors.white
            : Theme.of(context).primaryColor
        : widget.inactiveIconColor;
  }

  @override
  void initState() {
    super.initState();
    _setSelected(widget.tabs[widget.initialSelection].key);
  }

  _setSelected(UniqueKey key) {
    int selected = widget.tabs.indexWhere((tabData) => tabData.key == key);

    if (mounted) {
      setState(() {
        currentSelected = selected;
        _circleAlignX = -1 + (2 / (widget.tabs.length - 1) * selected);
        nextIcon = widget.tabs[selected].iconData;
      });
    }
  }

  _initAnimationAndStart(double from, double to) {
    _circleIconAlpha = 0;

    Future.delayed(Duration(milliseconds: I_ANIM_DURATION ~/ 5), () {
      setState(() {
        activeIcon = nextIcon;
      });
    }).then((_) {
      Future.delayed(Duration(milliseconds: (I_ANIM_DURATION ~/ 5 * 3)), () {
        setState(() {
          _circleIconAlpha = 1;
        });
      });
    });
  }

  void setPage(int page) {
    widget.onTabChangedListener(page);
    _setSelected(widget.tabs[page].key);
    _initAnimationAndStart(_circleAlignX, 1);

    setState(() {
      currentSelected = page;
    });
  }

  bool _isIPhoneX(MediaQueryData mediaQuery) {
    if (Platform.isIOS) {
      var size = mediaQuery.size;
      if (size.height >= 812.0 || size.width >= 812.0) {
        return true;
      }
    }
    return false;
  }

  bool _isIPhone() {
    if (Platform.isIOS) {
      return true;
    }
    return false;
  }

  @override
  Widget build(BuildContext context) {
    print("Is iPhobeX: " + _isIPhoneX(MediaQuery.of(context)).toString());
    return Stack(
      overflow: Overflow.visible,
      alignment: Alignment.bottomCenter,
      children: <Widget>[
        Container(
          padding: (_isIPhoneX(MediaQuery.of(context)))
          ? EdgeInsets.only(bottom: 20.0)
          : _isIPhone()
            ? EdgeInsets.only(bottom: 4.0)
            : EdgeInsets.zero,
          height: (_isIPhoneX(MediaQuery.of(context)))
            ? BAR_HEIGHT  + 20
            : _isIPhone()
              ? BAR_HEIGHT + 4
              : BAR_HEIGHT,
          decoration: BoxDecoration(color: barBackgroundColor, boxShadow: [
            BoxShadow(
                color: Colors.black12, offset: Offset(0, -1), blurRadius: 8)
          ]),
          child: Row(
            mainAxisSize: MainAxisSize.max,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: widget.tabs
                .map((t) => IndzoomTabItem(
                    uniqueKey: t.key,
                    selected: t.key == widget.tabs[currentSelected].key,
                    iconData: t.iconData,
                    title: t.title,
                    iconColor: inactiveIconColor,
                    textColor: textColor,
                    titleSize: t.titleSize,
                    callbackFunction: (uniqueKey) {
                      int selected = widget.tabs
                          .indexWhere((tabData) => tabData.key == uniqueKey);
                      widget.onTabChangedListener(selected);
                      _setSelected(uniqueKey);
                      _initAnimationAndStart(_circleAlignX, 1);
                    }))
                .toList(),
          ),
        ),
        Positioned.fill(
          top: - ((CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE) / 2) ,
          child: Container(
            child: AnimatedAlign(
              duration: Duration(milliseconds: I_ANIM_DURATION),
              curve: Curves.easeOut,
              alignment: Alignment(_circleAlignX, 1),
              child: Padding(
                // padding: const EdgeInsets.only(bottom: 15),
                padding: const EdgeInsets.only(bottom: 35),
                child: FractionallySizedBox(
                  widthFactor: 1 / widget.tabs.length,
                  child: GestureDetector(
                    onTap: widget.tabs[currentSelected].onclick,
                    child: Stack(
                      alignment: Alignment.center,
                      children: <Widget>[
                        SizedBox(
                          height:
                              CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE,
                          width:
                              CIRCLE_SIZE + CIRCLE_OUTLINE + SHADOW_ALLOWANCE,
                          child: ClipRect(
                              clipper: HalfClipper(),
                              child: Container(
                                child: Center(
                                  child: Container(
                                      width: CIRCLE_SIZE + CIRCLE_OUTLINE,
                                      height: CIRCLE_SIZE + CIRCLE_OUTLINE,
                                      decoration: BoxDecoration(
                                          color: Colors.white,
                                          shape: BoxShape.circle,
                                          boxShadow: [
                                            BoxShadow(
                                                color: Colors.black12,
                                                blurRadius: 8)
                                          ])),
                                ),
                              )),
                        ),
                        SizedBox(
                            height: ARC_HEIGHT,
                            width:
                              (_isIPhoneX(MediaQuery.of(context)))
                              ? ARC_WIDTH - 20
                              : _isIPhone()
                                ? ARC_WIDTH - 4
                                : ARC_WIDTH,
                            child: CustomPaint(
                              painter: HalfPainter(barBackgroundColor),
                            )),
                        SizedBox(
                          height: CIRCLE_SIZE,
                          width: CIRCLE_SIZE,
                          child: Container(
                            decoration: BoxDecoration(
                                shape: BoxShape.circle, color: circleColor),
                            child: Padding(
                              padding: const EdgeInsets.all(0.0),
                              child: AnimatedOpacity(
                                duration:
                                    Duration(milliseconds: I_ANIM_DURATION ~/ 5),
                                opacity: _circleIconAlpha,
                                child: Icon(
                                  activeIcon,
                                  size: 48.0,
                                  color: activeIconColor,
                                ),
                              ),
                            ),
                          ),
                        )
                      ],
                    ),
                  ),
                ),
              ),
            ),
          ),
        ),
        (_isIPhoneX(MediaQuery.of(context)))
        ?  Container(
            color: barBackgroundColor,
            height: 20.0,
            width: MediaQuery.of(context).size.width
          )
        : _isIPhone()
          ?  Container(
              color: barBackgroundColor,
              height: 4.0,
              width: MediaQuery.of(context).size.width
            )
          : Container(
              color: barBackgroundColor,
              height: 0,
              width: MediaQuery.of(context).size.width
          ),
      ],
    );
  }
}

class TabData {
  TabData({@required this.iconData, @required this.title, @required this.titleSize, this.onclick});

  IconData iconData;
  final UniqueKey key = UniqueKey();
  Function onclick;
  String title;
  double titleSize;
}
